@@ -1,9 +1,11 @@
 #!/usr/bin/env python2.5
 # -*- coding: utf-8 -*-
-""" Main-view UI for fMMS
+""" GUI.
 
-@author: Nick Leppänen Larsson <frals@frals.se>
-@license: GNU GPL
+fMMS - MMS for fremantle
+Copyright (C) 2010 Nick Leppänen Larsson <frals@frals.se>
+
+@license: GNU GPLv2, see COPYING file.
 """
 import os
 import time
@@ -21,6 +23,7 @@ from wappushhandler import PushHandler
 import fmms_config as fMMSconf
 import fmms_sender_ui as fMMSSenderUI
 import fmms_viewer as fMMSViewer
+import fmms_config_dialog as fMMSConfigDialog
 import controller as fMMSController
 import contacts as ContactH
 
@@ -28,8 +31,10 @@ import logging
 log = logging.getLogger('fmms.%s' % __name__)
 
 class fMMS_GUI(hildon.Program):
+	""" GUI class for the application. """
 
 	def __init__(self):
+		""" Initializes the GUI, creating all widgets. """
 		self.cont = fMMSController.fMMS_controller()
 		self.config = fMMSconf.fMMS_config()
 		self._mmsdir = self.config.get_mmsdir()
@@ -38,7 +43,7 @@ class fMMS_GUI(hildon.Program):
 		
 		self.osso_c = osso.Context("se.frals.fmms", self.config.get_version(), False)
 		self.osso_rpc = osso.Rpc(self.osso_c)
-      		self.osso_rpc.set_rpc_callback("se.frals.fmms","/se/frals/fmms","se.frals.fmms", self.cb_open_fmms, self.osso_c)
+		self.osso_rpc.set_rpc_callback("se.frals.fmms","/se/frals/fmms","se.frals.fmms", self.cb_open_fmms, self.osso_c)
 		
 		self.refreshlistview = True
 	
@@ -138,19 +143,21 @@ class fMMS_GUI(hildon.Program):
 		self.window.show_all()
 		self.add_window(self.window)
 
-	""" for taking a screenshot of the app to fake superfast load
-	 	
-	    inspired by Andrew Flegg and WimpWorks 
-	    @see http://maemo.org/api_refs/5.0/5.0-final/hildon/hildon-Additions-to-GTK+.html#hildon-gtk-window-take-screenshot """
-     	def take_ss(self, event=None, data=None):
+
+	def take_ss(self):
+		""" Takes a screenshot of the application used by hildon to show while loading.
+
+		inspired by Andrew Flegg and WimpWorks.
+		@see http://maemo.org/api_refs/5.0/5.0-final/hildon/hildon-Additions-to-GTK+.html#hildon-gtk-window-take-screenshot 
+		"""
 		if os.path.isfile("/home/user/.cache/launch/se.frals.fmms.pvr"):
 			gobject.timeout_add(10, hildon.hildon_gtk_window_take_screenshot, self.window, False)
 		
 		gobject.timeout_add(10, hildon.hildon_gtk_window_take_screenshot, self.window, True)
 
 
-	""" need this to always have the current path """
 	def cb_button_press(self, widget, event):
+		""" Used to keep track of the current selection in the treeview. """
 		try:
 			(self.curPath, tvcolumn, x, y) = self.treeview.get_path_at_pos(int(event.x), int(event.y))
 		except:
@@ -159,6 +166,7 @@ class fMMS_GUI(hildon.Program):
 
 
 	def cb_on_focus(self, widget, event):
+		""" Checks if the listview needs to be refreshed and takes screenshot. """
 		if self.refreshlistview == True:
 			t1 = time.clock()
 			hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
@@ -185,10 +193,10 @@ class fMMS_GUI(hildon.Program):
 
 
 	def cb_open_fmms(self, interface, method, args, user_data):
-		if method != 'open_mms' and method != 'open_gui' and method != 'send_via_service':
-			return
+		""" Determines what action should be done when a dbus-call is made. """
 		if method == 'open_mms':
 			filename = args[0]
+			self.refreshlistview = True
 			if self.cont.is_fetched_push_by_transid(filename):
 				hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
 				self.force_ui_update()
@@ -198,8 +206,6 @@ class fMMS_GUI(hildon.Program):
 			else:
 				return
 		elif method == 'open_gui':
-			# this shouldnt be needed as the on-focus cb should activate
-			#self.cb_on_focus(None, None)
 			return
 		elif method == 'send_mms':
 			log.info("launching sender with args: %s", args)
@@ -211,8 +217,11 @@ class fMMS_GUI(hildon.Program):
 			self.refreshlistview = False
 			ret = fMMSSenderUI.fMMS_SenderUI(withfile=args[0], subject=args[1], message=args[2]).run()
 			self.quit()
+		else:
+			return
 		
 	def create_menu(self):
+		""" Creates the application menu. """
 		menu = hildon.AppMenu()
 		
 		config = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
@@ -230,21 +239,21 @@ class fMMS_GUI(hildon.Program):
 		
 		return menu
 		
-		
 	def menu_button_clicked(self, button):
+		""" Determine what button was clicked in the app menu. """
 		buttontext = button.get_label()
 		if buttontext == "Configuration":
-			ret = self.create_config_dialog()
+			dialog = fMMSConfigDialog.fMMS_ConfigDialog(self.window)
 		elif buttontext == "About":
 			ret = self.create_about_dialog()
 	
-	
 	def new_mms_button_clicked(self, button):
+		""" Fired when the 'New MMS' button is clicked. """
 		self.refreshlistview = True
 		ret = fMMSSenderUI.fMMS_SenderUI(self.window).run()
 		
-		
 	def create_about_dialog(self):
+		""" Create and display the About dialog. """
 		dialog = gtk.AboutDialog()                                                 
 		dialog.set_name("fMMS")
 		fmms_logo = gtk.gdk.pixbuf_new_from_file("/opt/fmms/fmms.png")
@@ -256,280 +265,105 @@ class fMMS_GUI(hildon.Program):
 		dialog.set_website("http://mms.frals.se/")                                  
 		dialog.connect("response", lambda d, r: d.destroy())                      
 		dialog.show() 
- 
 
-	def create_config_dialog(self):
-		dialog = gtk.Dialog()
-		#dialog.set_transient_for(self.window)
-		dialog.set_title("Configuration")
-		
-		allVBox = gtk.VBox()
-		
-		self.active_apn_index = 0
-		
-		labelwidth = 16
-		
-		apnHBox = gtk.HBox()
-		apn_label = gtk.Label("APN:")
-		apn_label.set_width_chars(labelwidth)
-		self.selector = self.create_apn_selector()
-		self.apn = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
-		self.apn.set_selector(self.selector)
-		self.apn.set_active(self.active_apn_index)
-
-		apnHBox.pack_start(apn_label, False, True, 0)
-		apnHBox.pack_start(self.apn, True, True, 0)
-		
-		mmscHBox = gtk.HBox()
-		mmsc_label = gtk.Label("MMSC:")
-		mmsc_label.set_width_chars(labelwidth)
-		self.mmsc = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		mmsc_text = self.config.get_mmsc()
-		if mmsc_text != None:	
-			self.mmsc.set_text(mmsc_text)
-		else:
-			self.mmsc.set_text("http://")
-		mmscHBox.pack_start(mmsc_label, False, True, 0)
-		mmscHBox.pack_start(self.mmsc, True, True, 0)
-		
-		numberHBox = gtk.HBox()
-		number_label = gtk.Label("Your phonenumber:")
-		number_label.set_width_chars(labelwidth)
-		self.number = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		self.number.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_TELE)
-		number_text = self.config.get_phonenumber()
-		if number_text != None:
-			self.number.set_text(number_text)
-		else:
-			self.number.set_text("")
-		numberHBox.pack_start(number_label, False, True, 0)
-		numberHBox.pack_start(self.number, True, True, 0)
-		
-		imgwidthHBox = gtk.HBox()
-		imgwidth_label = gtk.Label("Resize image width:")
-		imgwidth_label.set_width_chars(labelwidth)
-		self.imgwidth = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		self.imgwidth.set_max_length(5)
-		#self.imgwidth_signal = self.imgwidth.connect('insert_text', self.insert_resize_cb)
-		self.imgwidth.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_NUMERIC)
-		imgwidth_text = self.config.get_img_resize_width()
-		if imgwidth_text != None:
-			self.imgwidth.set_text(str(imgwidth_text))
-		else:
-			self.imgwidth.set_text("")
-		imgwidthHBox.pack_start(imgwidth_label, False, True, 0)
-		imgwidthHBox.pack_start(self.imgwidth, True, True, 0)
-		
-		
-		expHBox = gtk.HBox()
-		exp_label = gtk.Label("Connection mode")
-		exp_label.set_width_chars(labelwidth)
-		""" 
-		    havoc = CONNMODE_UGLYHACK = 1
-		    polite = CONNMODE_ICDSWITCH = 2
-		    rude = CONNMODE_FORCESWITCH = 3
-		"""
-		self.havocbutton = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		self.havocbutton.set_label("Havoc")
-		self.rudebutton = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT, self.havocbutton)
-		self.rudebutton.set_label("Rude")
-		self.icdbutton = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT, self.havocbutton)
-		self.icdbutton.set_label("Polite")
-		""" set the correct button active """
-		self.connmode_setactive()
-		
-		expHBox.pack_start(exp_label, False, True, 0)
-		expHBox.pack_start(self.icdbutton, True, True, 0)
-		expHBox.pack_start(self.rudebutton, True, True, 0)
-		expHBox.pack_start(self.havocbutton, True, True, 0)
-		
-		allVBox.pack_start(apnHBox, False, False, 0)
-		allVBox.pack_start(mmscHBox, False, False, 0)
-		allVBox.pack_start(numberHBox, False, False, 0)
-		allVBox.pack_start(imgwidthHBox, False, False, 0)
-		allVBox.pack_end(expHBox, False, False, 0)
-		
-		allVBox.show_all()
-		dialog.vbox.add(allVBox)
-		dialog.add_button("Save", gtk.RESPONSE_APPLY)
-		ret = dialog.run()
-		ret2 = self.config_menu_button_clicked(ret)
-		dialog.destroy()
-
-
-	""" selector for apn """
-	def create_apn_selector(self):
-		selector = hildon.TouchSelector(text = True)
-		apnlist = self.config.get_gprs_apns()
-		currval = self.config.get_apn_nicename()
-		# Populate selector
-		i = 0
-		for apn in apnlist:
-			if apn != None:
-				if apn == currval:
-					self.active_apn_index = i
-				i += 1	
-				# Add item to the column 
-				selector.append_text(apn)
-			
-		selector.center_on_selected()
-		selector.set_active(0, i)
-		# Set selection mode to allow multiple selection
-		selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
-		return selector
-		
-	def connmode_option(self):
-		""" 
-		    havoc = CONNMODE_UGLYHACK = 1
-		    polite = CONNMODE_ICDSWITCH = 2
-		    rude = CONNMODE_FORCESWITCH = 3
-		"""
-		if self.havocbutton.get_active():
-			return fMMSconf.CONNMODE_UGLYHACK
-		elif self.icdbutton.get_active():
-			return fMMSconf.CONNMODE_ICDSWITCH
-		elif self.rudebutton.get_active():
-			return fMMSconf.CONNMODE_FORCESWITCH
-		
-	def connmode_setactive(self):
-		if self.config.get_connmode() == fMMSconf.CONNMODE_UGLYHACK:
-			self.havocbutton.set_active(True)
-		elif self.config.get_connmode() == fMMSconf.CONNMODE_ICDSWITCH:
-			self.icdbutton.set_active(True)
-		elif self.config.get_connmode() == fMMSconf.CONNMODE_FORCESWITCH:
-			self.rudebutton.set_active(True)
-	
-		
-	def config_menu_button_clicked(self, action):
-		if action == gtk.RESPONSE_APPLY:
-			log.info("%s", self.apn.get_selector().get_current_text())
-			ret_setapn = self.config.get_apnid_from_name(self.apn.get_selector().get_current_text())
-			if ret_setapn != None:
-				self.config.set_apn(ret_setapn)
-				log.info("Set apn to: %s" % ret_setapn)
-				ret = self.config.set_mmsc(self.mmsc.get_text())
-				log.info("Set mmsc to %s" % self.mmsc.get_text())
-				self.config.set_phonenumber(self.number.get_text())
-				log.info("Set phonenumber to %s" % self.number.get_text())
-				self.config.set_img_resize_width(self.imgwidth.get_text())
-				log.info("Set image width to %s" % self.imgwidth.get_text())
-				self.config.set_connmode(self.connmode_option())
-				log.info("Set connection mode %s" % self.connmode_option())				
-				banner = hildon.hildon_banner_show_information(self.window, "", "Settings saved")
-				return 0
-			else:
-				log.info("Set mmsc to %s" % self.mmsc.get_text())
-				self.config.set_phonenumber(self.number.get_text())
-				log.info("Set phonenumber to %s" % self.number.get_text())
-				self.config.set_img_resize_width(self.imgwidth.get_text())
-				log.info("Set image width to %s" % self.imgwidth.get_text())
-				banner = hildon.hildon_banner_show_information(self.window, "", "Could not save APN settings. Did you enter a correct APN?")
-				banner.set_timeout(5000)
-				return -1
-
-
-	""" add each item to our liststore """
 	def add_buttons_liststore(self):
-			icon_theme = gtk.icon_theme_get_default()
-			
-			pushlist = self.cont.get_push_list()
-			
-			primarytxt = self.cont.get_primary_font().to_string()
-			primarycolor = self.cont.get_primary_color().to_string()
-			highlightcolor = self.cont.get_active_color().to_string()
-			secondarytxt = self.cont.get_secondary_font().to_string()
-			secondarycolor = self.cont.get_secondary_color().to_string()
-			for varlist in pushlist:
-                                mtime = varlist['Time']
-                                # TODO: Remove date if date == today
-                                # TODO: get locale format?
-                                mtime = self.cont.convert_timeformat(mtime, "%Y-%m-%d | %H:%M")
+		""" Adds all messages to the liststore. """
+		icon_theme = gtk.icon_theme_get_default()
 
-				fname = varlist['Transaction-Id']
-				direction = self.cont.get_direction_mms(fname)
+		pushlist = self.cont.get_push_list()
 
-				isread = False
-				if self.cont.is_mms_read(fname):
-					isread = True
-				
+		primarytxt = self.cont.get_primary_font().to_string()
+		primarycolor = self.cont.get_primary_color().to_string()
+		highlightcolor = self.cont.get_active_color().to_string()
+		secondarytxt = self.cont.get_secondary_font().to_string()
+		secondarycolor = self.cont.get_secondary_color().to_string()
+		for varlist in pushlist:
+			mtime = varlist['Time']
+			# TODO: Remove date if date == today
+			# TODO: get locale format?
+			mtime = self.cont.convert_timeformat(mtime, "%Y-%m-%d | %H:%M")
+
+			fname = varlist['Transaction-Id']
+			direction = self.cont.get_direction_mms(fname)
+
+			isread = False
+			if self.cont.is_mms_read(fname):
+				isread = True
+
+			try:
+				sender = varlist['From']
+				sender = sender.replace("/TYPE=PLMN", "")
+			except:
+				sender = "0000000"
+
+			if direction == fMMSController.MSG_DIRECTION_OUT:
+				sender = self.cont.get_mms_headers(varlist['Transaction-Id'])
+				sender = sender['To'].replace("/TYPE=PLMN", "")
+
+			senderuid = self.ch.get_uid_from_number(sender)
+			photo = icon_theme.load_icon("general_default_avatar", 48, 0)
+			if senderuid != None:
+				sender = self.ch.get_displayname_from_uid(senderuid)
+				phototest = self.ch.get_photo_from_uid(senderuid, 48)
+				if phototest != None:
+					photo = phototest
+
+			if direction == fMMSController.MSG_DIRECTION_OUT:
+				icon = icon_theme.load_icon("chat_replied_sms", 48, 0)
+			elif self.cont.is_fetched_push_by_transid(fname) and isread:
+				icon = icon_theme.load_icon("general_sms", 48, 0)
+			else:
+				icon = icon_theme.load_icon("chat_unread_sms", 48, 0)
+
+			try:
+				headerlist = self.cont.get_mms_headers(fname)
+				description = cgi.escape(headerlist['Description'])
+			except:
 				try:
-					sender = varlist['From']
-					sender = sender.replace("/TYPE=PLMN", "")
+					description = varlist['Subject']
 				except:
-					sender = "0000000"
-				
-				if direction == fMMSController.MSG_DIRECTION_OUT:
-					sender = self.cont.get_mms_headers(varlist['Transaction-Id'])
-					sender = sender['To'].replace("/TYPE=PLMN", "")
-				
-				sendername = self.ch.get_name_from_number(sender)
-				photo = icon_theme.load_icon("general_default_avatar", 48, 0)
-				if sendername != None:
-					sender = sendername
-					phototest = self.ch.get_photo_from_name(sendername, 48)
-					if phototest != None:	
-						photo = phototest
-				
-				if direction == fMMSController.MSG_DIRECTION_OUT:
-					icon = icon_theme.load_icon("chat_replied_sms", 48, 0)
-				elif self.cont.is_fetched_push_by_transid(fname) and isread:
-					icon = icon_theme.load_icon("general_sms", 48, 0)
-				else:
-					icon = icon_theme.load_icon("chat_unread_sms", 48, 0)
-					
-				try:
-					headerlist = self.cont.get_mms_headers(fname)
-					description = cgi.escape(headerlist['Description'])
-				except:
-					try:
-						description = varlist['Subject']
-					except:
-						description = ""
-						
-				primarytext = ' <span font_desc="%s" foreground="%s"><sup>%s</sup></span>' % (secondarytxt, secondarycolor, mtime)
-				secondarytext = '\n<span font_desc="%s" foreground="%s">%s</span>' % (secondarytxt, secondarycolor, description)
-				if not isread and direction == fMMSController.MSG_DIRECTION_IN:
-					sender = '<span foreground="%s">%s</span>' % (highlightcolor, sender)
-				stringline = "%s%s%s" % (sender, primarytext, secondarytext)
-				self.liststore.append([icon, stringline, photo, fname, sender])
+					description = ""
+
+			primarytext = ' <span font_desc="%s" foreground="%s"><sup>%s</sup></span>' % (secondarytxt, secondarycolor, mtime)
+			secondarytext = '\n<span font_desc="%s" foreground="%s">%s</span>' % (secondarytxt, secondarycolor, description)
+			if not isread and direction == fMMSController.MSG_DIRECTION_IN:
+				sender = '<span foreground="%s">%s</span>' % (highlightcolor, sender)
+			stringline = "%s%s%s" % (sender, primarytext, secondarytext)
+			self.liststore.append([icon, stringline, photo, fname, sender])
 
-	
-	""" lets call it quits! """
 	def quit(self, *args):
+		""" Quits the application. """
 		gtk.main_quit()
 	
-	
-	""" forces ui update, kinda... god this is AWESOME """
 	def force_ui_update(self):
+		""" Force a UI update if events are pending. """
 		while gtk.events_pending():
 			gtk.main_iteration(False)
 		
-		
-	""" delete push message """
 	def delete_push(self, fname):
+		""" Deletes the given push message. """
 		self.cont.delete_push_message(fname)
 		
-	
-	""" delete mms message (eg for redownload) """
 	def delete_mms(self, fname):
+		""" Deletes the given MMS message. """
 		self.cont.delete_mms_message(fname)
 
-	
-	""" delete push & mms """
 	def delete_push_mms(self, fname):
+		""" Deletes both the MMS and the PUSH message. """
 		try:
 			self.cont.wipe_message(fname)
 			#banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Message deleted")
-			self.refreshlistview = True
 		except Exception, e:
 			log.exception("%s %s", type(e), e)
 			#raise
 			banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Failed to delete message.")
-			self.refreshlistview = True
 
-
-	""" action on delete contextmenu click """
 	def liststore_delete_clicked(self, widget):
+		""" Shows a confirm dialog when Delete menu is clicked.
+
+		Deletes the message if the user accepts the dialog.
+
+		"""
 		if self.curPath == None:
 			return
 			
@@ -549,14 +383,13 @@ class fMMS_GUI(hildon.Program):
 		if ret == 1:
 			log.info("deleting %s", filename)
 			self.delete_push_mms(filename)
-			#self.liststore.remove(miter)
+			self.liststore.remove(miter)
 		dialog.destroy()
-		self.refreshlistview = True
 		return
 	
 
-	""" long press on image creates this """
 	def liststore_mms_menu(self):
+		""" Creates the context menu and shows it. """
 		menu = gtk.Menu()
 		menu.set_property("name", "hildon-context-sensitive-menu")
 
@@ -568,9 +401,8 @@ class fMMS_GUI(hildon.Program):
 		menu.show_all()
 		return menu
 
-
-	""" show the selected mms """		
 	def show_mms(self, treeview, path):
+		""" Shows the message at the current selection in the treeview. """
 		# Show loading indicator
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
 		self.force_ui_update()
@@ -581,22 +413,26 @@ class fMMS_GUI(hildon.Program):
 		# the 4th value is the transactionid (start counting at 0)
 		transactionid = model.get_value(miter, 3)
 		
+		if not self.cont.is_mms_read(transactionid) and not self.cont.get_direction_mms(transactionid) == fMMSController.MSG_DIRECTION_OUT:
+			self.refreshlistview = True
+		
 		try:
 			viewer = fMMSViewer.fMMS_Viewer(transactionid)
 		except Exception, e:
 			log.exception("%s %s", type(e), e)
 			#raise
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
-		self.refreshlistview = True
-
-
+		
 	def run(self):
+		""" Run. """
 		self.window.show_all()
 		gtk.main()
-				
+
+
 if __name__ == "__main__":
 	try:
 		app = fMMS_GUI()
 		app.run()
 	except:
-		log.exception("General failure.")
\ No newline at end of file
+		log.exception("General failure.")
+		raise
\ No newline at end of file
